import glob
import os
from pathlib import Path
from turtle import st
from typing import Dict, List, Optional, Union

import numpy as np
import torch
from torch.utils import data
from natsort import natsorted

from .basedataset import GradSLAMDataset

class AwadhpuriDataset(GradSLAMDataset):
    def __init__(
            self, 
            config_dict,
            basedir,
            sequence,
            stride: Optional[int] = None, 
            start: Optional[int] = 0, 
            end: Optional[int] = -1, 
            desired_height: Optional[int] = 480, 
            desired_width: Optional[int] = 640, 
            load_embeddings: Optional[bool] = False,
            embedding_dir: Optional[str] = "embeddings",
            embedding_dim: Optional[int] = 512,
            **kwargs
        ):
            self.input_folder = os.path.join(basedir, sequence)
            self.pose_path = os.path.join(self.input_folder, "poses.txt")
            super().__init__(
                  config_dict, 
                  stride=stride, 
                  start=start, 
                  end=end, 
                  desired_height=desired_height, 
                  desired_width=desired_width, 
                  load_embeddings=load_embeddings, 
                  embedding_dir=embedding_dir, 
                  embedding_dim=embedding_dim, 
                  **kwargs)

    def parse_list(self, filepath, skiprows=0):
        """ read list data """
        data = np.loadtxt(filepath, delimiter=',',
                          dtype=np.unicode_, skiprows=skiprows)
        return data
    
    def pose_matrix_from_quaternion(self, pvec):
        """ convert 4x4 pose matrix to (t, q) """
        from scipy.spatial.transform import Rotation

        # POSE indices = tx=0, ty=1, tz=2, qw=3,qx=4,qy=5,qz=6
        quat = np.array([pvec[4], pvec[5], pvec[6], pvec[3]])

        pose = np.eye(4)
        pose[:3, :3] = Rotation.from_quat(quat).as_matrix()
        pose[:3, 3] = pvec[:3]
        return pose
    
    def get_filepaths(self):
        color_paths = natsorted(glob.glob(f"{self.input_folder}/rgb/left_*.png"))
        depth_paths = natsorted(glob.glob(f"{self.input_folder}/depth/disp_*.png"))
        embedding_paths = None

        print("color_paths shape:", len(color_paths))
        print("depth_paths shape:", len(depth_paths))

        return color_paths, depth_paths, embedding_paths

    def load_poses(self):
        pose_data = self.parse_list(self.pose_path, skiprows=1)
        print("POSE_DATA:", pose_data)
        pose_vecs = pose_data[:, 1:].astype(np.float64)

        pose_indices = pose_data[:, 0].astype(np.int32)
        print("POSE_INDICES:", pose_indices.shape)

        poses = []
        for i in range(pose_indices.shape[0]):
             c2w = self.pose_matrix_from_quaternion(pose_vecs[i])
             c2w = torch.from_numpy(c2w).float()
             poses += [c2w]

        return poses

    def read_embedding_from_file(self, embedding_file_path):
        embedding = torch.load(embedding_file_path, map_location="cpu")
        return embedding.permute(0, 2, 3, 1)